#include "QN8027_DRV.h"
#include "..\IIC\IIC.h"

unsigned char QN8027_ReadReg(unsigned char RegBase)
{
	unsigned char RegValue;
	
	IIC_Start();
	IIC_SendOneByte(QN8027_WR_Operation);
	if(IIC_RecvACK_Error())
	{
		IIC_Stop();
		return 0;
	}
	IIC_SendOneByte(RegBase);
	if(IIC_RecvACK_Error())
	{
		IIC_Stop();
		return 0;
	}
	//IIC_Stop();
	IIC_Start();
	IIC_SendOneByte(QN8027_RD_Operation);
	if(IIC_RecvACK_Error())
	{
		IIC_Stop();
		return 0;
	}
	RegValue = IIC_RecvOneByte();
	IIC_SendACK(NACK);
	IIC_Stop();

	return RegValue;
}

bit QN8027_WriteReg(unsigned char RegBase, unsigned char RegValue)
{
	IIC_Start();
	IIC_SendOneByte(QN8027_WR_Operation);
	if(IIC_RecvACK_Error())
	{
		IIC_Stop();
		return QN8027_WR_ERROR;
	}
	IIC_SendOneByte(RegBase);
	if(IIC_RecvACK_Error())
	{
		IIC_Stop();
		return QN8027_WR_ERROR;
	}
	IIC_SendOneByte(RegValue);
	if(IIC_RecvACK_Error())
	{
		IIC_Stop();
		return QN8027_WR_ERROR;
	}
	IIC_Stop();
	
	return QN8027_WR_SUCCESS;
}

void QN8027_Init(void)
{
	QN8027_WriteReg(0x00, 0x81);				//Reset all registers to their default values.
	Delay_us(3000);
	//QN8027_WriteReg(0x03, 0x10);			//Use crystal on XTAL1/XTAL2. 
	QN8027_WriteReg(0x04, 0x50);				//XTAL frequency: 12MHz, input impedance for both L/R channels: 5KOhms, TX input buffer gain: 0dB, TX digital gain: 0dB 
	QN8027_WriteReg(0x00, 0x41);				//FSM calibration. 
	QN8027_WriteReg(0x00, 0x01);				//FSM calibration completed. 
	Delay_us(3000);
	//QN8027_WriteReg(0x10, 0x4B);			//PA output power target is 0.62*PA_TRGT+71dBu. Valid values are 20-75.  
	QN8027_WriteReg(0x18, 0xE4);				//Improve SNR. (This is an undocumented register)
	QN8027_WriteReg(0x1B, 0xF0);				//Increase maximum RF output power. (This is an undocumented register, and it really works.) 
	QN8027_WriteReg(0x02, 0xB9);				//Disable RF PA off function.
}

void QN8027_SetFrequency(unsigned short Freq)	//The parameter "Freq" is your desired frequency multiplied by 10.
{
	unsigned short REG_CH;
	unsigned char REG_SYSTEM;

	if((Freq >= 760)&&(Freq <= 1080))						//76MHz~108MHz
	{
		REG_SYSTEM = QN8027_ReadReg(0x00);
		REG_SYSTEM &= 0xDF;
		QN8027_WriteReg(0x00, REG_SYSTEM);				//Disable transmission.

		REG_SYSTEM &= 0xFC;
		REG_CH = (Freq << 1) - 1520;							//Channel frequency is (76+CH*0.05) MHz
		REG_SYSTEM |= (REG_CH >> 8);
		QN8027_WriteReg(0x00, REG_SYSTEM);				//Set the highest 2 bits of 10-bit channel index.
		QN8027_WriteReg(0x01, (REG_CH&0x00FF));
	
		REG_SYSTEM |= 0x20;
		QN8027_WriteReg(0x00, REG_SYSTEM);				//Enable transmission.
	}
}

unsigned short QN8027_GetCurrentFrequency(void)	//The return value is the actual frequency value multiplied by 10.
{
	unsigned short Freq;

	Freq = QN8027_ReadReg(0x00) & 0x03;
	Freq = (((Freq << 8)| QN8027_ReadReg(0x01)) >> 1) + 760;

	return Freq;
}

unsigned char QN8027_CheckID()
{
	unsigned char REG_CID, ChipID;

	REG_CID = QN8027_ReadReg(0x06);
	if((REG_CID >> 4) == 0x04)									//QN8027 is detected.
	{
		ChipID = (REG_CID + 1) << 4;							//Retrieve chip major revision number.
		REG_CID = (QN8027_ReadReg(0x05) & 0x03) + 1;
		ChipID |= REG_CID;												//Attach chip minor revision number.
		return ChipID;
	}
	
	return 0;																		//Return chip revision number if QN8027 is detected, otherwise return 0.
}
