#include "SST89x5xxRD2.H"
#include "uart.h"


unsigned char WrBufIdx, RdBufIdx;
unsigned char UartBuf[UART_BUF_SIZE];


void UartInit(void)
{
	WrBufIdx = 0;
	RdBufIdx = 0;

	TR1 = 0;									//Disable Timer 1.
	PCON |= 0x80;							//Double Baud Rate.
	T2CON &= 0xCF;						//Use Timer 1 overflow for UART transmit and receive clock.

	TMOD &= 0x0F;
	TMOD |= 0x20;							//Timer 1 works in Mode 2 (8-bit Auto-Reload).

	TH1 = 0xFA;								//n = 256 - [fosc/(baud1612)]
	TL1 = 0xFA;								//e.g. n = 256 - 11059200/(96001612) = 250 = 0xFA

	SCON = 0x50;							//Serial Port Control Register. Serial Port works in Mode 1.
									
	//IP = 0x10;							//Interrupt Priority. Serial Port has the highest interrupt priority.
	
	ES = 1;										//Enable serial interrupt.
	TR1 = 1;									//Enable Timer 1.
}

/* Send one byte */ 
void UartSendChar(unsigned char Character)
{
	SBUF = Character;
	while(TI == 0);						//Wait until serial transmission occurs.
	TI = 0;										//Transmit interrupt flag must be cleared by software.
}

/* Send a string */
void UartSendString(unsigned char *Str)
{
	while(*Str)
	{
		UartSendChar(*Str);
		Str++;
	}
}

/* Check if there is data received. Return the length of the string if data is received, otherwise return 0. */
unsigned char IsUartReceived(void)
{
	unsigned char StrLen;
	
	EA = 0;										//Global Interrupt Disable
	if(WrBufIdx == RdBufIdx)
		StrLen = 0;
	else if(WrBufIdx > RdBufIdx)
		StrLen = WrBufIdx - RdBufIdx;
	else if(WrBufIdx < RdBufIdx) 
		StrLen = UART_BUF_SIZE - WrBufIdx + RdBufIdx;
	EA = 1;										//Global Interrupt Enable
	
	return StrLen;
}

/* Read one byte from buffer and return the number of bytes left. */
unsigned char UartReadChar(unsigned char *Character)
{
	unsigned char BytesToRead;

	EA = 0;										//Global Interrupt Disable
	if(WrBufIdx > RdBufIdx)
	{
		*Character = UartBuf[RdBufIdx];
		RdBufIdx++;
		BytesToRead = WrBufIdx - RdBufIdx;
	}
	else if(WrBufIdx < RdBufIdx)
	{
		*Character = UartBuf[RdBufIdx];
		RdBufIdx++;
		if(RdBufIdx == UART_BUF_SIZE)
			RdBufIdx = 0;
		BytesToRead = UART_BUF_SIZE - RdBufIdx + WrBufIdx;
	}
	else
	{
		*Character = 0;
		BytesToRead = 0;
	}

	EA = 1;						    		//Global Interrupt Enable
	return BytesToRead;
}

void UartISR(void) interrupt 4 using 1
{
	if(RI)										//Receive Interrupt Flag
	{								
		ES = 0;									//Disable serial interrupt.
		UartBuf[WrBufIdx] = SBUF;
		RI = 0;									//Receive interrupt flag must be cleared by software.
		
		if(WrBufIdx >= UART_BUF_SIZE - 1)
			WrBufIdx = 0;
		else
			WrBufIdx++;
		if(WrBufIdx == RdBufIdx)
			RdBufIdx = WrBufIdx + 1;

		ES = 1;									//Enable serial interrupt.
	}
}
